@@ -5,7 +5,7 @@ bool
 LogUnit::openLog(const char * filename)
 {
 	if (logFile.is_open()) {
-		cerr << APPNAME 
+		cerr << APPNAME
 			<< ": opening a new Log file, while another is already open"
 			<< endl;
 		logFile.close();
@@ -18,9 +18,6 @@ LogUnit::openLog(const char * filename)
 void
 LogUnit::closeLog()
 {
-	if (logFile.is_open()) {
+	if (logFile.is_open())
 		logFile.close();
-	} else {
-		cerr << APPNAME << ": closing Log file, while is already closed" << endl;
-	}
 }
