@@ -21,6 +21,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <string.h>
+#include <unistd.h>
+#include <sys/socket.h>
 #include <png.h>
 #include "const.h"
 
